#ifndef __NQS_GENERIC_TBLMSGS_HEADER
#define __NQS_GENERIC_TBLMSGS_HEADER

/*
 * nqs_generic/all-systems/tblmsgs.h
 * Structure for storing tables of messages
 */

#include <nqs_generic/license.h>
  /* NQS license information */

struct TAG_msg_table
{
  long lCode;
    /* Code to test against */
  char *szName;
    /* name of the value in lCode */
  char *szPrefix;
    /* Message to output before outputing user-supplied info */
  char *szSuffix;
    /* Message to output after outputting user-supplied info */
  char *szExtraSuffix;
    /* Second message to output after outputting user-supplied info */
};

typedef struct TAG_msg_table msg_table;

#define MSGTBLE(a, b, c, d) \
  { a, #a, b, c, d }

#define MSGTBLE_2(a, b, c, d, e) \
  { a, #a, c, d, e}, \
  { b, #b, c, d, e}

#define MSGTBLE_END \
  { 0, NULL, NULL, NULL, NULL }

#endif /* __NQS_GENERIC_TBLMSGS_HEADER */
